﻿// Copyright 2018 Consorsbank
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// tag::Using[]
using System;
using System.Collections.Generic;
using Grpc.Core;
using System.Threading.Tasks;
using System.Threading;
// end::Using[]

namespace TestClient
{
    // tag::StreamObserver[]
    public abstract class StreamObserver<TClient, TRequest, TReply> where TClient :
        ClientBase<TClient>
    {
        // Define delegates
        public delegate void ResultListener(TReply reply); // <1>
        public delegate AsyncServerStreamingCall<TReply> 
            Call(TRequest request, CancellationToken token); // <2>

        private readonly TClient _client; 
        private readonly Call _caller;

        protected StreamObserver(TClient client, Call caller)
        {
            this._client = client;
            this._caller = caller;
        }

        public async Task Stream(TRequest request, ResultListener listener,
            CancellationToken token = default(global::System.Threading.CancellationToken))
        {
            try
            {
                using (AsyncServerStreamingCall<TReply> call = _caller(request, token)) // <3>
                {
                    var responseStream = call.ResponseStream; // <4>
                    while (!token.IsCancellationRequested && await responseStream.MoveNext()) // <5>
                    {
                        {
                            TReply reply = responseStream.Current; // <6>
                            listener?.Invoke(reply); // <7>
                        }
                    }
                }
            }
            catch (RpcException e)
            {
                if (e.StatusCode!=StatusCode.Cancelled)
                {
                    Console.WriteLine(e); // <8>
                    throw;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e); // <8>
                throw;
            }
        }
    }
    // end::StreamObserver[]
}
